# ---------------------------------------------------------------------------
# Convert an APM-based mission to one that can be read into DJI Pilot PE
# Description: This script is to create a KML that can be read by the Pilot app
# from a Mission Planner or Fire Fly mission.  Output has the same file name with
# "_dji" appended.
# By Jake Slyder, jslyder@blm.gov, 20200103
# ---------------------------------------------------------------------------

import time
import sys
import arcpy

if sys.version_info[0] == 2:
    import Tkinter as tk
    import tkFileDialog as fdialog

else: #Run through Python 3 interpreter
    import tkinter as tk
    import tkinter.filedialog as fdialog


startTime = time.time()

root = tk.Tk()
root.withdraw()

inTxtPath = fdialog.askopenfilenames(parent=root,initialdir="/",title='Select MP/FF Mission Text File', filetypes = (("TXT files","*.txt"),("all files","*.*")))


for i in range(len(inTxtPath)):
    curPath = inTxtPath[i]
    inParse = curPath.split("/")
    print("Starting conversion of of %s" %inParse[-1])
    fName = inParse[-1].replace(".txt","_dji.kml")
    del inParse[-1]
    outPath = "\\".join(inParse)
    outKML = outPath + "\\" + fName

    f = open(curPath,'r')
    file = open(outKML, "w")

    #Read the lines
    allLines = f.readlines()
    initLine = allLines[0]
    homeLine = allLines[1]
    flightLines = allLines[2:]
    f.close()
    #Create list of lists where each element of l is a list of x,y,z coords
    l = []

    #Set a default speed.  If one is found in the flight line commmands, go ahead and update it.
    speed = 5.0

    for line in flightLines:
        col = line.split("\t")
        command = int(col[3])
        latitude = float(col[8])
        longitude = float(col[9])
        altitude = float(col[10])
        if command == 16:
            coords = [longitude, latitude, altitude]
            l.append(coords)
        if command == 178:
            pntSpeed = float(col[5])
            if pntSpeed > speed:
                speed = float(col[5])

    #Write the header for the kml
    file.write('<?xml version="1.0" encoding="UTF-8"?>\n\n<kml xmlns="http://www.opengis.net/kml/2.2">\n  <Document xmlns="">\n    <name>Waypoint1</name>\n    <open>1</open>\n    <ExtendedData xmlns:mis="www.dji.com">\n      <mis:type>Waypoint</mis:type>\n    </ExtendedData>\n    <Style id="waylineGreenPoly">\n      <LineStyle>\n        <color>FF0AEE8B</color>\n        <width>6</width>\n      </LineStyle>\n    </Style>\n    <Style id="waypointStyle">\n      <IconStyle>\n        <Icon>\n          <href>https://cdnen.dji-flighthub.com/static/app/images/point.png</href>\n        </Icon>\n      </IconStyle>\n    </Style>\n    <Folder>\n      <name>Waypoints</name>\n      <description>Waypoints in the Mission.</description>\n')
    #Next, iterate through wayspoints to create placemark elements

    wplist = ''
    counter = 1
    for i in l:
        longi = str(i[0])
        lat = str(i[1])
        alt = str(i[2])
        wpName = "Waypoint" + str(counter)
        line ='      <Placemark>\n        <name>%s</name>\n        <visibility>1</visibility>\n        <description>Waypoint</description>\n        <styleUrl>#waypointStyle</styleUrl>\n        <ExtendedData xmlns:mis="www.dji.com">\n          <mis:useWaylineAltitude>false</mis:useWaylineAltitude>\n          <mis:heading>0</mis:heading>\n          <mis:turnMode>Clockwise</mis:turnMode>\n          <mis:gimbalPitch>0.0</mis:gimbalPitch>\n        </ExtendedData>\n        <Point>\n          <altitudeMode>relativeToGround</altitudeMode>\n          <coordinates>%s,%s,%s</coordinates>\n        </Point>\n      </Placemark>\n' %(wpName,longi,lat,alt)
        file.write(line)
        #Create a string with the list of coordiantes for writing the Wayline
        cl = '%s,%s,%s '%(longi,lat,alt)
        wplist = wplist + cl
        counter += 1

    altLine = l[0][2]
    #Finally, add the wayline to the d ocument
    wayline='    </Folder>\n      <Placemark>\n        <name>Wayline</name>\n        <description>Wayline</description>\n        <visibility>1</visibility>\n        <ExtendedData xmlns:mis="www.dji.com">\n          <mis:altitude>%s</mis:altitude>\n          <mis:autoFlightSpeed>%s</mis:autoFlightSpeed>\n          <mis:actionOnFinish>GoHome</mis:actionOnFinish>\n          <mis:headingMode>Auto</mis:headingMode>\n          <mis:gimbalPitchMode>ControlledByRC</mis:gimbalPitchMode>\n        </ExtendedData>\n        <styleUrl>#waylineGreenPoly</styleUrl>\n        <LineString>\n          <tessellate>1</tessellate>\n          <altitudeMode>relativeToGround</altitudeMode>\n          <coordinates>%s</coordinates>\n        </LineString>\n      </Placemark>\n  </Document>\n</kml>\n'%(altLine,speed,wplist)
    file.write(wayline)
    file.close()
    print("%s successfully converted to %s" %(curPath,outKML))

    outLine = outKML.replace(".kml",".shp")
    #print(outLine)
    tmpPointName = fName.replace(".kml","TEMPPOINTS.shp")
    #print(tmpPointName)
    sr = arcpy.SpatialReference("WGS 1984")
    pntTemp = arcpy.CreateFeatureclass_management(outPath, tmpPointName, "POINT", "", "", "", sr)
    #messages.addMessage("Temp point copy created")
    cur = arcpy.da.InsertCursor(pntTemp, ["SHAPE@X","SHAPE@Y"])
    #messages.addMessage("cursor created")
    for i in range(len(l)):
        cur.insertRow((l[i][0], l[i][1]))
    #messages.addMessage("cursor completed")
    del cur
    f.close()

    outLine = outKML.replace(".kml",".shp")
    #convert way points to line
    arcpy.PointsToLine_management(pntTemp, outLine)
    #Delete temp shp file
    arcpy.Delete_management(pntTemp)

print ('The script took {0} seconds !'.format(time.time() - startTime))
